--##########################################################################--
--[[----------------------------------------------------------------------]]--
-- key/mouse pressed/released callbacks
--[[----------------------------------------------------------------------]]--
--##########################################################################--
function love.keypressed(key)
  for _,v in ipairs(controls.keys) do
  
    if v.const == key then
      -- set key's press stats
      v.pressed = true
      v.timer:start()
      v.timer_is_active = true
      
      if v.press_action ~= nil and v.press_enabled and v.enabled then
        v.press_action()
      end
      
      -- update press rate value
      local time = v.rate_timer:time_elapsed()/1000
      
      if time > v.rate_threshold then
        v.total_time = 0
        v.counter = 0
        time = v.rate_threshold
      end
      v.total_time = v.total_time + time
      v.counter = v.counter + 1
      v.rate_timer:start()
      
      local rate = v.counter / v.total_time
      v.rate = rate
      
    end
    
  end
  
end

function love.keyreleased(key)
  for _,v in ipairs(controls.keys) do
  
    if v.const == key then
      v.released = true
      v.timer_is_active = false
      
      if v.release_action ~= nil and v.release_enabled and v.enabled then
        v.release_action()
      end
    end
    
  end
end

function love.mousepressed(x, y, button)
  for _,v in ipairs(controls.buttons) do
  
    if v.const == button then
      -- set button's press stats
      v.pressed = true
      v.x, v.y = mouse:get_vals()
      v.timer:start()
      v.timer_is_active = true
      
      if v.press_action ~= nil and v.press_enabled and v.enabled then
        v.press_action()
      end
      
      -- update press rate value
      local time = v.rate_timer:time_elapsed()/1000
      
      if time > v.rate_threshold then
        v.total_time = 0
        v.counter = 0
        time = v.rate_threshold
      end
      v.total_time = v.total_time + time
      v.counter = v.counter + 1
      v.rate_timer:start()
      
      local rate = v.counter / v.total_time
      v.rate = rate
      
    end
    
  end
  
end

function love.mousereleased(x, y, button)
  for _,v in ipairs(controls.buttons) do
  
    if v.const == button then
      v.released = true
      v.timer_is_active = false
      
      if v.release_action ~= nil and v.release_enabled and v.enabled then
        v.release_action()
      end
    end
    
  end
end


--##########################################################################--
--[[----------------------------------------------------------------------]]--
-- Initialize controls
--[[----------------------------------------------------------------------]]--
--##########################################################################--
local controls = keyboard:new()

-- valid keys
controls:add_key('quit', 'escape')
controls:add_key('activate', 'return')
controls:add_key('up', 'up')
controls:add_key('down', 'down')
controls:add_key('reset', 'r')
controls:add_key('pause', 'p')
controls:add_key('attack', 'lshift')

controls:add_button('click', 'l')

controls:disable_all()

--------------------------------------------------------------------------------
-- default layout
controls['quit'].enabled = true
controls['quit'].press_action = function() love.event.push('quit') end

controls:save_layout('default')
controls:disable_all()

--------------------------------------------------------------------------------
-- first_state layout
controls['quit'].enabled = true
controls['quit'].press_action = function() love.event.push('quit') end
controls['activate'].enabled = true
controls['activate'].press_action = function() menu:activate() end
controls['click'].enabled = true
controls['click'].press_action = function() menu:activate() end

controls:save_layout('first_state')
controls:disable_all()

--------------------------------------------------------------------------------
-- second_state layout
controls['quit'].enabled = true
controls['quit'].press_action = function() love.event.push('quit') end
controls['click'].enabled = true
controls['click'].press_action = function() 
                                   local x = controls.click.x
                                   local y = controls.click.y
                                   --print('click!', x, y) 
                                 end
                                 
controls['reset'].enabled = true
controls['reset'].press_action = function() 
                                   love.load(ARGS)
                                 end
                                 
controls['pause'].enabled = true
controls['pause'].press_action = function() 
                                   if master_timer.is_stopped then
                                     master_timer:start()
                                   else
                                     master_timer:stop()
                                   end
                                 end
                                 
controls['attack'].enabled = true
controls['attack'].press_action = function() hero:shoot() end

controls:save_layout('second_state')
controls:disable_all()


return controls

--[[ sample usage:
controls = keyboard:new()

controls:add_key('up',         'w',      'navigation')
controls:add_key('left',       'a',      'navigation')
controls:add_key('down',       's',      'navigation')
controls:add_key('right',      'd',      'navigation')
controls:add_key('shoot',      'rshift', 'hero')
controls:add_key('escape',     'escape', 'menu')
controls:add_key('activate',   'return', 'menu')
controls:add_key('fullscreen', 'f'             )
controls:add_key('debug',      'b'             )
controls:add_key('next',       'n'             )
controls:add_key('quit',       'escape'             )
controls:add_key('reset',      'r'             )

controls:disable_all()

------------------------------------------------------------------------------
-- menu_state layout
controls['up'].enabled = true
controls['up'].press_action = function() menu.menu:up() end
controls['down'].enabled = true
controls['down'].press_action = function() menu.menu:down() end
controls['activate'].enabled = true
controls['activate'].press_action = function() menu.menu:activate() end
controls['fullscreen'].enabled = true
controls['fullscreen'].press_action = function() love.graphics.toggleFullscreen() end
controls['escape'].enabled = true
controls['escape'].press_action = function() love.event.push('quit') end
controls['next'].enabled = true
controls['next'].press_action = function() invaders:load_next_state() end

controls:save_layout('menu')
controls:disable_all()

------------------------------------------------------------------------------
-- intro_state layout
controls['activate'].enabled = true
controls['activate'].press_action = function() intro_state.continue() end
controls['escape'].enabled = true
controls['escape'].press_action = function() love.event.push('quit') end
controls['next'].enabled = true
controls['next'].press_action = function() invaders:load_next_state() end

controls:save_layout('intro')
controls:disable_all()

------------------------------------------------------------------------------
-- level0_state layout
controls['escape'].enabled = true
controls['escape'].press_action = function() love.event.push('quit') end
controls['debug'].enabled = true
controls['debug'].press_action = function() DEBUG = not DEBUG end
controls['next'].enabled = true
controls['next'].press_action = function() invaders:load_next_state() end
controls['reset'].enabled = true
controls['reset'].press_action = function() invaders.load() end

controls:save_layout('level0')
controls:disable_all()

return controls
]]--































