require 'invaders_functions'
require 'object_loader'
require 'state_loader'
require 'font_loader'

function love.load(arg)
  -- GLOBALS -------------------------------------------------------------------
  lg = love.graphics
  lk = love.keyboard
  ARGS = arg
  DEBUG = true
  
  function lg.resize_window(width, height)
    SCR_WIDTH, SCR_HEIGHT = width, height
    
    -- mouse_input scaling depends on window size
    mouse = mouse_input:new()
    lg.setMode(width, height, FULLSCREEN, true)
  end
  
  -- screen
  SCR_WIDTH  = arg[2] or 800
  SCR_HEIGHT = arg[3] or 600
  FULLSCREEN = arg[4] and true
  RANDCOLOUR = true
  TILE_WIDTH = 40
  TILE_HEIGHT = 40
  CELL_WIDTH = 256   -- collider cell width
  CELL_HEIGHT = 256
  
  -- colors
  C_WHITE = {255, 255, 255, 255}
  C_BLACK = {0, 0, 0, 255}
  C_RED = {255, 0, 0, 255}
  C_BORDER = {0, 0, 0, 80}
  CL1_RED = {240, 77, 99}
  CL1_TEAL = {88, 222, 246}
  CL1_RED_D = {171, 9, 15, 255}
  CL1_TEAL_D = {114, 178, 173, 255}
  
  -- channels
  RED, GREEN, BLUE, ALPHA = 1, 2, 3, 4
  
  ------------------------------------------------------------------------------

  -- find user's resolution
  love.graphics.setMode(0, 0)
  local max_width = love.graphics.getWidth()
  local max_height = love.graphics.getHeight()
  love.graphics.setMode(SCR_WIDTH, SCR_HEIGHT, false, false)
  local resolutions = get_supported_resolutions(max_width, max_height)
  
  load_fonts()
  load_objects()
  load_states()
  
  -- keeps track of time for the game
  master_timer = master_timer:new()
  
  -- initialize control layouts
  love.mouse.setVisible(false)
  mouse = mouse_input:new()
  controls = require('conf_controls')
  
  -- set states of the game
  invaders = state_manager:new()
  invaders:add_state(first_state)
  invaders:add_state(second_state)
  invaders:add_state(level_state)
  invaders:load_state(first_state, resolutions)

end

function love.update(dt)
	-- debug stuff
	if DEBUG then
		if camera then
			if     lk.isDown("c") then
				camera.debug = true
			else
				camera.debug = false
			end
		end
		if hero then
			if     lk.isDown("h") then
				hero.debug = true
			else
				hero.debug = false
			end
		end
		if collider then
			if     lk.isDown("t") then
				collider.debug = true
			else
				collider.debug = false
			end
		end
		
	end

  master_timer:update(dt)
  if master_timer.is_stopped then dt = 0 end
  controls:update(dt)
  mouse:update(dt)
  invaders.current_state.update(dt)
end

function love.draw()
  invaders.current_state.draw()
  mouse:draw()
  --lg.setFont(courier_small)
  lg.setColor(255, 0, 0, 255)
end
