
--##########################################################################--
--[[----------------------------------------------------------------------]]--
-- bullet object
--[[----------------------------------------------------------------------]]--
--##########################################################################--
local bullet = {}
bullet.table = BULLET
bullet.parent = nil         -- gun that sent the bullet
bullet.owner = nil          -- object that triggered the gun
bullet.map = nil
bullet.isdead = false

-- position
bullet.mpoint = nil
bullet.pos = nil
bullet.dir = nil

-- movement
bullet.collider = nil
bullet.fire_speed = 500
bullet.vel = nil
bullet.depth = 0
bullet.max_depth = 1


local bullet_mt = { __index = bullet }
function bullet:new(position, direction, velocity, map)

	local vel = bullet.fire_speed * direction + velocity
	local mpoint = map_point:new(position, map)
	
  return setmetatable({ pos = position,
                        dir = direction,
                        vel = vel,
                        map = map,
                        mpoint = mpoint,
                        collider = collider}, bullet_mt)
end

function bullet:init(position, direction, velocity)
	self.vel = bullet.fire_speed * direction + velocity
	self.depth = bullet.depth
	self:set_position(position)
	self:set_direction(direction)
	collider:add_object(self.mpoint, self)
	self.isdead = false
	
	if not self.mpoint.tile.type.walkable then
		self:die()
	end
end

function bullet:set_parent(obj)
	self.parent = obj
end
function bullet:set_owner(obj)
	self.owner = obj
end
function bullet:set_position(pos)
	self.pos = pos
	self.mpoint:update_position(pos)
end
function bullet:set_direction(dir)
	self.direction = dir
end

function bullet:die()
	self.isdead = true
end

------------------------------------------------------------------------------
function bullet:update(dt)
	-- move point
	local newpos = self.pos + self.vel * dt
	local point = self.mpoint	
	point:set_position(newpos)
	point:update(dt)
	c, n, p = point:get_collision_data()
	
	-- collision with tile
	if c then
		self.vel = get_bounce_velocity(self.vel, n, 0.8, 0.9)
		self.depth = self.depth + 1
		point:update_position(p)
		newpos = p
		if self.depth > self.max_depth then
			self:die()
		end
	end
	
	self.pos = newpos
	collider:update_object(point.cdata.id)
end

------------------------------------------------------------------------------
function bullet:draw()
end

return bullet































