
--##########################################################################--
--[[----------------------------------------------------------------------]]--
-- gun object
--[[----------------------------------------------------------------------]]--
--##########################################################################--
local gun = {}
gun.table = GUN
gun.debug = false
gun.parent = nil
gun.pos = nil
gun.dir = nil

gun.num_bullets = 1000
gun.bullets = nil					-- active bullets
gun.deadpool = nil        -- pool of ready to use bullets

local gun_mt = { __index = gun }
function gun:new(position, direction, map)
	-- create bullets with dummy values
	local ammo = {}
	for i=1,gun.num_bullets do
		ammo[i] = bullet:new(vector2:new(100,100), NORMAL[UP], VECT_ZERO, map)
	end
	
  return setmetatable({ pos = position,
                        dir = direction,
  											map = map,
                        deadpool = ammo,
                        bullets = {}}, gun_mt)
end

-- pos = vector
function gun:set_position(pos)
	self.pos = pos
end
function gun:get_position()
	return self.pos
end

-- dir = normalized vector
function gun:set_direction(dir)
	self.dir = dir
end
function gun:get_direction()
	return self.dir
end

function gun:set_parent(obj)
	self.parent = obj
end

function gun:shoot()
	if #self.deadpool == 0 then return end
	
	local bullet = self.deadpool[#self.deadpool]
	self.deadpool[#self.deadpool] = nil
	self.bullets[#self.bullets + 1] = bullet
	
	bullet:set_parent(self)
	bullet:set_owner(self.parent)
	bullet:init(self.pos, self.dir, self.parent:get_velocity() * DSCALE)
end

------------------------------------------------------------------------------
function gun:update(dt)
	local bullets = self.bullets
	for i=#bullets,1,-1 do
		local bullet = bullets[i]
		bullet:update(dt)
		if bullet.isdead then
			table.remove(bullets, i)
			self.deadpool[#self.deadpool+1] = bullet
			collider:remove_object(bullet.mpoint.cdata.id)
		end
	end
end

------------------------------------------------------------------------------
function gun:draw()
	local bullets = self.bullets
	for i=1,#bullets do
		local pos = bullets[i].pos
		lg.setColor(0, 255, 0, 255)
		lg.setPoint(5, "smooth")
		lg.point(pos.x, pos.y)
	end
end

------------------------------------------------------------------------------
function gun:draw_debug()
	local pos = self.pos
	local dir = self.dir
	local length = 10
	local endpos = pos + length * dir
	lg.setColor(0,255,0,150)
	lg.setLine(3)
	lg.line(pos.x, pos.y, endpos.x, endpos.y)
end

return gun
































