
--##########################################################################--
--[[----------------------------------------------------------------------]]--
-- mouse_input object
--[[----------------------------------------------------------------------]]--
--##########################################################################--
local mouse_input = {}
mouse_input.table = MOUSE_INPUT
mouse_input.scale = 1.5                  -- factor to scale mouse input by
mouse_input.reference_pos = nil
mouse_input.screen_pos = nil

mouse_input.x = 0
mouse_input.y = 0
mouse_input.width = SCR_WIDTH
mouse_input.height = SCR_HEIGHT

local mouse_input_mt = { __index = mouse_input }
function mouse_input:new()
  local pos = vector2:new(0.5*SCR_WIDTH, 0.5*SCR_HEIGHT)
  love.mouse.setPosition(pos:get_vals())
  local width, height = SCR_WIDTH, SCR_HEIGHT
  
  return setmetatable({ reference_pos = pos,
                        screen_pos = pos,
                        width = width,
                        height = height}, mouse_input_mt)
end

------------------------------------------------------------------------------
function mouse_input:set_sensitivity(scale_factor)
  self.scale = scale_factor
end

------------------------------------------------------------------------------
function mouse_input:set_boundary(x, y, width, height)
  self.x, self.y, self.width, self.height = x, y, width, height
end

------------------------------------------------------------------------------
function mouse_input:get_position()
  return self.screen_pos
end

------------------------------------------------------------------------------
function mouse_input:get_vals()
  return self.screen_pos.x, self.screen_pos.y
end

------------------------------------------------------------------------------
function mouse_input:set_position(pos)
	self.screen_pos = pos
end

------------------------------------------------------------------------------
function mouse_input:update(dt)
  -- scale diffence between reference and new position
  local last_pos = self.reference_pos
  local new_pos = vector2:new(love.mouse.getPosition())
  
  self.screen_pos = self.screen_pos + (new_pos - last_pos) * self.scale
  love.mouse.setPosition(self.reference_pos:get_vals())
  
  -- limit screen position to within boundary
  local x, y = self.screen_pos:get_vals()
  if     x < self.x then 
    self.screen_pos.x = self.x 
  elseif x > self.x + self.width then 
    self.screen_pos.x = self.x + self.width 
  end
  if     y < self.y then 
    self.screen_pos.y = self.y 
  elseif y > self.y + self.height then 
    self.screen_pos.y = self.y + self.height 
  end
  
end

------------------------------------------------------------------------------
function mouse_input:draw()
  lg.setColor(C_BLACK)
  lg.setPoint(3, 'smooth')
  lg.point(self.screen_pos:get_vals())
end

return mouse_input



