
--##########################################################################--
--[[----------------------------------------------------------------------]]--
-- tile_chunk object
--[[----------------------------------------------------------------------]]--
--##########################################################################--
local tile_chunk = {}
tile_chunk.table = TILE_CHUNK
tile_chunk.parent = nil
tile_chunk.x = nil
tile_chunk.y = nil
tile_chunk.tiles = nil
tile_chunk.width = nil     -- in # of tiles
tile_chunk.height = nil
tile_chunk.image = nil     -- contains spritesheet
tile_chunk.quads = nil     -- quads that match spritesheet
tile_chunk.max_sprites = 1000
tile_chunk.sprite_batch = nil
tile_chunk.is_current = false   -- is false if the chunk requires a refresh or
                                -- an update

local tile_chunk_mt = { __index = tile_chunk }
function tile_chunk:new()
  return setmetatable({tiles = {}}, tile_chunk_mt)
end

------------------------------------------------------------------------------
function tile_chunk:set_tiles(tiles)
  self.tiles = tiles
  self.height = #tiles
  self.width = #tiles[1]
end

function tile_chunk:set_parent(parent)
  self.parent = parent
end

-- sets image used as a spritesheet, and its cooresponding quads
function tile_chunk:set_image(img, quads)
  self.image = img
  self.quads = quads
end

------------------------------------------------------------------------------
function tile_chunk:init_sprite_batch()
  self.sprite_batch = love.graphics.newSpriteBatch(self.image, self.max_sprites)
end

------------------------------------------------------------------------------
function tile_chunk:refresh_batch()

  local batch = self.sprite_batch
  batch:clear()
  
  for j=1,self.height do
    for i=1,self.width do
      local tile = self.tiles[j][i]
      batch:addq(tile.type.quad, tile.batch_x, tile.batch_y)
    end
  end
  
  self.is_current = true
end

------------------------------------------------------------------------------
function tile_chunk:update(dt)
  if not self.is_current then
    self:refresh_batch()
  end
end

------------------------------------------------------------------------------
function tile_chunk:draw()  
  lg.draw(self.sprite_batch, self.x, self.y)
end

return tile_chunk









