local level_state = state:new()
level_state.label = 'level_state'
level_state.keyboard_layout = 'second_state'


--##########################################################################--
--[[----------------------------------------------------------------------]]--
--      LOAD
--[[----------------------------------------------------------------------]]--
--##########################################################################--
function level_state.load(map, camera)
	lg.setBackgroundColor(0,0,0,255)
  -- collisions
  local width = map.width * TILE_WIDTH
  local height = map.height * TILE_HEIGHT
  collider = collider:new(width, height, camera)
  
  -- set up hero
  local hero_pos = vector2:new(3010, 2500)
	hero = hero:new(hero_pos, map, camera)
	camera:set_target(hero_pos, true)
	
	pos = vector2:new(1005, 1005)

	-- collision test objects
  points = {}
  speed = 30
  local density = 0.05  -- points per tile
  local num = math.floor(density * map.width * map.height)
  for i=1,num do
  	local bdr = 2 * TILE_WIDTH
  	local minx, maxx = 0 + bdr, map.width * TILE_WIDTH - bdr
  	local miny, maxy = 0 + bdr, map.height * TILE_HEIGHT - bdr
  	m = map_point:new(vector2:new(math.random(minx, maxx), math.random(miny, maxy)), map)
  	collider:add_object(m)
  	m.vel = vector2:new(math.random(-100, 100), math.random(-100, 100))
  	m.vel = m.vel:normalize()
  	m.vel = m.vel * math.random(speed)
  	local min, max = 10, #map.palette[1]
  	
  	local rand = math.random(min, max)
  	local min_size, max_size = 2,8
  	local size = min_size + math.floor((max_size-min_size) * (rand-min) / (max - min))
  	m.size = size
  	
  	local color = map.palette[1][rand]
  	m.color = color
  	points[#points+1] = m
  	
  	tile = m:_get_tile(m.pos)
  	if not tile.type.walkable then
  		collider:remove_object(m.cdata.id)
  	end
  	
  	m.test = true
  end
  
  boxes = {}
	num = 800
	for i=1,num do
		local bdr = 150
		local minx, maxx = 0 + bdr, map.width * TILE_WIDTH - bdr
		local miny, maxy = 0 + bdr, map.height * TILE_HEIGHT - bdr
		local x, y = math.random(minx, maxx),  math.random(miny, maxy)
		local width = math.random(10,15)
		local height = width
		if math.random() > 0.85 then
			width = math.random(50,100)
			height = width
		end
		
		local box = bbox:new(x, y, width, height)
		local points = {vector2:new(box.x, box.y),
									  vector2:new(box.x + box.width, box.y + box.height)}
		local body = map_body:new(points, map)
		box.body = body
		box.body:set_origin(vector2:new(box.x, box.y))
		local ok = box.body:check_tiles()
		
		if ok then
			box.oldx, box.oldy = x, y
			local vel = vector2:new(math.random(-100, 100), math.random(-100, 100))
			vel = vel:normalize()
			box.vel = vel * math.random(speed)
			collider:add_object(box)
			boxes[#boxes + 1] = box
			
			local min, max = 10, #map.palette[1]
			local rand = math.random(min, max)
			local color = map.palette[1][rand]
			box.color = color
			
			box.test = true
			
			if box.width > 50 then
				box.vel = box.vel / 2
			end
			
			box.vel = vector2:new(0, 0)
		end
	end
	
	for i=1,#boxes do
		local box = boxes[i]
		local objects = collider:get_collisions(box.cdata.id)
		if #objects > 0 then
			collider:remove_object(box.cdata.id)
		end
	end
 
  
  -- joystick test
  joystick = love.joystick.open(1)
  print(love.joystick.getName(1))
  print(love.joystick.getNumAxes(1))
  
  shoot_timer = timer:new(25)
  shoot_timer:start()
  killcount = 0
end


--##########################################################################--
--[[----------------------------------------------------------------------]]--
--      UPDATE
--[[----------------------------------------------------------------------]]--
--##########################################################################--
fc = 0
function level_state.update(dt)
	dt = math.min(1/30, dt)
	fc = fc + 1
	if love.keyboard.isDown('lshift') or love.joystick.isDown(1, 1 ) then
		if not shoot_timer:isrunning() then
			hero:shoot()
			shoot_timer:start()
		end
	end
	
	if love.keyboard.isDown('up') then
		for i=1,5 do
			local bdr = 150
			local minx, maxx = 0 + bdr, map.width * TILE_WIDTH - bdr
			local miny, maxy = 0 + bdr, map.height * TILE_HEIGHT - bdr
			local x, y = math.random(minx, maxx),  math.random(miny, maxy)
			local width = math.random(10,15)
			local height = width
			if math.random() > 0.85 then
				width = math.random(50,100)
				height = width
			end
			
			local box = bbox:new(x, y, width, height)
			local points = {vector2:new(box.x, box.y),
											vector2:new(box.x + box.width, box.y + box.height)}
			local body = map_body:new(points, map)
			box.body = body
			box.body:set_origin(vector2:new(box.x, box.y))
			local ok = box.body:check_tiles()
			
			if ok then
				box.oldx, box.oldy = x, y
				local vel = vector2:new(math.random(-100, 100), math.random(-100, 100))
				vel = vel:normalize()
				box.vel = vel * math.random(speed)
				collider:add_object(box)
				boxes[#boxes + 1] = box
				
				local min, max = 10, #map.palette[1]
				local rand = math.random(min, max)
				local color = map.palette[1][rand]
				box.color = color
				
				box.test = true
				
				if box.width > 50 then
					box.vel = box.vel / 2
				end
				
				box.vel = vector2:new(0, 0)
			end
		end
	end
	

	-- joystick yest
	if joystick then
		ax1, ax2 = love.joystick.getAxes(1)
		if math.abs(ax1) > 0 or math.abs(ax2) > 0 then
			jdir = vector2:new(ax1, ax2)
			jpower = math.min(jdir:mag(), 1)
		else
			jdir = hero.direction
			jpower = 0
		end
		jdir = jdir:normalize()
	end
	

  if love.keyboard.isDown("w") then
  end
  if love.keyboard.isDown("a") then
  end
  if love.keyboard.isDown("s") then
  end
  if love.keyboard.isDown("d") then
  end
  if love.keyboard.isDown("s") then
  	dt = dt/10
  end
  
  map:update(dt)
  camera:update(dt)
  hero:update(dt)
  
  points = {}
  rects = {}
  local added = {}
  for i=1, #collider.cells_in_view do
  	local cell = collider.cells_in_view[i]
  	for i,id in pairs(cell.contents) do
  		local object = collider.objects[id]
  		if object.table == MAP_POINT and object.test then
  			points[#points+1] = object
  		end
  		if object.table == BBOX and object.test then
  			if not added[id] then
					rects[#rects + 1] = object
					added[id] = true
  			end
  		end
  	end
  end
  
  if love.keyboard.isDown('down') then
		for i=1,5 do
			if #rects > 0 then
				local box = rects[#rects]
				rects[#rects] = nil
				collider:remove_object(box.cdata.id)
			end
			
			if #points > 0 then
				local p = points[#points]
				points[#points] = nil
				collider:remove_object(p.cdata.id)
			end
		end
	end
  
  for i=1,#points do
  	local m = points[i]
  	local x = m:get_x()
  	local y = m:get_y()
  	x = x + m.vel.x * dt
  	y = y + m.vel.y * dt
  	m:set_position(vector2:new(x, y))
  	m:update(dt)
  	
  	c, n, p = m:get_collision_data()
		if c then
			m:update_position(p)
			m.vel = get_bounce_velocity(m.vel, n, 0.5, 0.9)
			m.vel = m.vel:normalize()
			m.vel = m.vel * math.random(speed)
		end
  	
  	if m.cdata then
  		collider:update_object(m.cdata.id)
  		if math.random() > 0.99 then
  			m.vel = vector2:new(math.random(-100, 100), math.random(-100, 100))
  			m.vel = m.vel:normalize() * math.random(speed)
				local v = hero.point.pos - vector2:new(m:get_x(), m:get_y())
				local dist_sq = v:mag_sq()
				local radius = 150
				local thresh
				if dist_sq < radius * radius then
					thresh = 0.1
				else
					thresh = 0.9
				end
				if math.random() > thresh then
					v = v:normalize()
					if m.size <= 6 then v = -v end
					m.vel = v
					m.vel = m.vel:normalize()
					m.vel = m.vel * math.random(2*speed)
				end
  		end
  	end
  	
  end
  
  local mw, mh = map.width * TILE_WIDTH, map.height * TILE_HEIGHT
  for i=1,#rects do
  	local box = rects[i]
  	local x, y = box:get_x(), box:get_y()
  	
  	if box.vel then
			local newx = x + box.vel.x * dt
			local newy = y + box.vel.y * dt
			
			local newpos = vector2:new(newx, newy)
			box.body:set_position(newpos)
			box.body:update(dt)
			local c, n, p, offset = box.body:get_collision_data()
			if c then
				local new_pos = box.body:get_position() + offset
				local success, safe_pos = box.body:update_position(new_pos)
				if not success then
					new_pos = safe_pos
				end
				
				newx, newy = new_pos.x, new_pos.y
				
				box.vel = get_bounce_velocity(box.vel, n, 0.9, 0.3)
			end
			
			box.oldx, box.oldy = box.x, box.y
			box.x = newx
			box.y = newy
			collider:update_object(box.cdata.id)
			
  	end
  	
  end
  
  collider:update_object(id)
  collider:update(dt)
  
  -- resolve test collisions
  objects = collider:get_collisions(hero.bbox.cdata.id)
  for _,v in ipairs(objects) do
  	local vel = hero.point.pos - vector2:new(v:get_x(), v:get_y())
  	vel = -vel:normalize() * math.random(20, 50)
  	vel = vel + hero.point.vel * DSCALE
  	v.vel =  vel
  end
  
  local collided = {}
  local world_box = bbox:new(0, 0, map.width*TILE_WIDTH, map.height*TILE_HEIGHT)
  for i=1,#rects do
  	local box = rects[i]
  	local objects = collider:get_collisions(box.cdata.id)
  	for j=1,#objects do
  		local obj = objects[j]
  		if obj.table == MAP_POINT and obj == true then
  			local cbox = vector2:new(box.x + 0.5 * box.width, box.y + 0.5 * box.height)
  			local p = vector2:new(obj:get_x(), obj:get_y())
  			local dir = p - cbox
  			dir = dir:normalize()
  			obj.vel = dir * math.random(2 * speed) + box.vel
  			
  		elseif obj.table == MAP_POINT and obj.cdata.parent then
  			if obj.cdata.parent.table == BULLET then
  				-- collided with bullet
  				collider:remove_object(box.cdata.id)
  				obj.cdata.parent:die()
  				killcount = killcount + 1
  				
  				if box.width >= 50 then
  					local bdr = 250
  					local blast = bbox:new(box.x - bdr, box.y - bdr, box.width + 2*bdr, box.height + 2*bdr)
  					
  					if world_box:contains(blast) then
							collider:add_object(blast)
							local b_objects = collider:get_collisions(blast.cdata.id)
							collider:remove_object(blast.cdata.id)
							for k=1,#b_objects do
								local obj = b_objects[k]
								if obj.test then
									local cx, cy = box.x + 0.5 * box.width, box.y + 0.5 * box.height
									local vel = vector2:new(obj:get_x() - cx, obj:get_y() - cy)
									vel = vel:normalize()
									obj.vel = vel * (200 + math.random(4 * speed))
								end
							end
						end
						
						if blast:contains(hero.bbox) then
							local cx, cy = blast.x + 0.5*blast.width, blast.y + 0.5*blast.height
							local cpos = vector2:new(cx, cy)
							local dist = vector2:dist(hero.pos, cpos)
							local r = 1 - (dist / math.max(blast.width, blast.height))
							local dir = hero.pos - cpos
							dir = dir:normalize()
							local minforce = 75
							local maxforce = 150
							local force = (minforce + (maxforce-minforce) * r) * dir
							hero.point:add_force(force, 150)
						end
						
  					
  					local bvel = obj.cdata.parent.vel
  					local minx, maxx = box.x, box.x + box.width
  					local miny, maxy = box.y, box.y + box.height
  					local cx, cy = box.x + 0.5 * box.width, box.y + 0.5 * box.height
  					local num = math.floor((box.width * box.height) / (15*20))
  					if num > 25 then num = 25 end
  					local i
  					for j=1,num do
  						local w = math.random(10,15)
  						local x = math.random(minx, maxx - w)
  						local y = math.random(miny, maxy - w)
  						local b = bbox:new(x, y, w, w)
  					
							local points = {vector2:new(b.x, b.y),
														vector2:new(b.x + b.width, b.y + b.height)}
							local body = map_body:new(points, map)
							b.body = body
							b.body:set_origin(vector2:new(b.x, b.y))
  						
  						b.oldx, b.oldy = x, y
  						local vel = vector2:new(x + 0.5*w - cx, y + 0.5*w - cy)
							vel = (vel:normalize() + bvel:normalize() + box.vel:normalize())/3
							vel = vel:normalize()
							b.vel = vel * (135 + math.random(4*speed))
							collider:add_object(b)
							boxes[#boxes + 1] = b
							
							local min, max = 10, #map.palette[1]
							local rand = math.random(min, max)
							local color = map.palette[1][rand]
							b.color = color
							
							b.test = true
  					end
  				else
  					local bvel = obj.cdata.parent.vel
  					local minx, maxx = box.x, box.x + box.width
  					local miny, maxy = box.y, box.y + box.height
  					local cx, cy = box.x + 0.5 * box.width, box.y + 0.5 * box.height
  					local num = math.random(7, 15)
  					for j=1,num do
  						local x = math.random(minx, maxx)
  						local y = math.random(miny, maxy)
  						local pos = vector2:new(x, y)
  						m = map_point:new(pos, map)
  						local tile = m:_get_tile(pos)
  						
  						if tile.type.walkable then
  							collider:add_object(m)
  							local vel = vector2:new(x - cx, y - cy)
  							vel = (vel:normalize() + bvel:normalize() + box.vel:normalize())/3
  							vel = vel:normalize()
  							m.vel = vel * (200 + math.random(4*speed))
  							local min, max = 10, #map.palette[1]
  	
								local rand = math.random(min, max)
								local min_size, max_size = 2,8
								local size = min_size + math.floor((max_size-min_size) * (rand-min) / (max - min))
								m.size = size
								
								local color = map.palette[1][rand]
								m.color = color
								m.test = true
								
								if math.random() > 0.01 then
									m.timer = timer:new(2000)
									m.timer:start()
								end
  						end
  					end
  				end
  				
  				break
  			end
  		elseif obj.table == BBOX and obj.test then
  			if not collided[box.cdata.id] and not collided[obj.cdata.id] then
					local cbox1 = vector2:new(box.x + 0.5 * box.width, box.y + 0.5 * box.height)
					local cbox2 = vector2:new(obj.x + 0.5 * obj.width, obj.y + 0.5 * obj.height)
					local dir = cbox2 - cbox1
					dir = dir:normalize()
					local speed = math.random(5,10)
					local bsq, osq = box.width * box.height, obj.width * obj.height
					local mass_ratio = bsq / (bsq + osq)
					local avg_vel = mass_ratio * box.vel + (1 - mass_ratio) * obj.vel
					
					obj.vel = dir * speed + avg_vel
					box.vel = -dir * speed + avg_vel
					
					collided[box.cdata.id] = true
					collided[obj.cdata.id] = true
				end
  		end
  		
  	end
  end
  
end

-- joystick test
function love.joystickpressed(joystick, button)
	if button == 1 then -- ps3 square
		hero:shoot()
	end
end
   
--##########################################################################--
--[[----------------------------------------------------------------------]]--
--     DRAW
--[[----------------------------------------------------------------------]]--
--##########################################################################--

love.filesystem.setIdentity( "screen_test" )
function level_state.draw()

	camera:set_target(hero:get_camera_target())
	
  camera:set()
  map:draw()
  
  hero:draw()
  
  camera:draw()
  collider:draw()
  
  if not collider.debug then
		for i=1,#rects do
			local box = rects[i]
			lg.setColor(box.color)
			lg.rectangle("fill", box.x, box.y, box.width, box.height)
		end
		
		for i=1,#points do
			local point = points[i]
			if point.timer then
				local p = point.timer:progress()
				local a = 255 * (1-p*p)
				lg.setColor(point.color[1], point.color[2], point.color[3], a)
				if p == 1 then
					collider:remove_object(point.cdata.id)
				end
			else
				lg.setColor(point.color)
			end
			
			if point.cdata then
				lg.setPoint(point.size, "smooth")
				lg.point(point.cdata.x, point.cdata.y)
			end
		end
  end
  
  camera:unset()
  
  lg.setColor(0,255, 150, 255)
  --lg.print(love.timer:getFPS(), 20, 20)
  lg.print("Boxes Dismantled: "..killcount, 20, 20)
  
  
end

return level_state




















